package dp.背包模型;

import java.util.Scanner;

/**
 * 宠物小精灵之收服
 * 总时间限制: 1000ms 内存限制: 65536kB
 * 描述
 * 宠物小精灵是一部讲述小智和他的搭档皮卡丘一起冒险的故事。
 * 一天，小智和皮卡丘来到了小精灵狩猎场，里面有很多珍贵的野生宠物小精灵。小智也想收服其中的一些小精灵。
 * 然而，野生的小精灵并不那么容易被收服。对于每一个野生小精灵而言，小智可能需要使用很多个精灵球才能收服它，而在收服过程中，
 * 野生小精灵也会对皮卡丘造成一定的伤害（从而减少皮卡丘的体力）。当皮卡丘的体力小于等于0时，
 * 小智就必须结束狩猎（因为他需要给皮卡丘疗伤），而使得皮卡丘体力小于等于0的野生小精灵也不会被小智收服。当小智的精灵球用完时，狩猎也宣告结束。
 * 我们假设小智遇到野生小精灵时有两个选择：收服它，或者离开它。如果小智选择了收服，
 * 那么一定会扔出能够收服该小精灵的精灵球，而皮卡丘也一定会受到相应的伤害；如果选择离开它，那么小智不会损失精灵球，皮卡丘也不会损失体力。
 * 小智的目标有两个：主要目标是收服尽可能多的野生小精灵；如果可以收服的小精灵数量一样，
 * 小智希望皮卡丘受到的伤害越小（剩余体力越大），因为他们还要继续冒险。
 * 现在已知小智的精灵球数量和皮卡丘的初始体力，已知每一个小精灵需要的用于收服的精灵球数目和它在被收服过程中会对皮卡丘造成的伤害数目。
 * 请问，小智该如何选择收服哪些小精灵以达到他的目标呢？
 * 输入
 * 输入数据的第一行包含三个整数：N(0 < N < 1000)，M(0 < M < 500)，K(0 < K < 100)，分别代表小智的精灵球数量、皮卡丘初始的体力值、野生小精灵的数量。
 * 之后的K行，每一行代表一个野生小精灵，包括两个整数：收服该小精灵需要的精灵球的数量，以及收服过程中对皮卡丘造成的伤害。
 * 输出
 * 输出为一行，包含两个整数：C，R，分别表示最多收服C个小精灵，以及收服C个小精灵时皮卡丘的剩余体力值最多为R。
 * 样例输入
 * 样例输入1：
 * 10 100 5
 * 7 10
 * 2 40
 * 2 50
 * 1 20
 * 4 20
 * <p>
 * 样例输入2：
 * 10 100 5
 * 8 110
 * 12 10
 * 20 10
 * 5 200
 * 1 110
 * 样例输出
 * 样例输出1：
 * 3 30
 * <p>
 * 样例输出2：
 * 0 100
 * 提示
 * 对于样例输入1：小智选择：(7,10) (2,40) (1,20) 这样小智一共收服了3个小精灵，皮卡丘受到了70点伤害，剩余100-70=30点体力。所以输出3 30
 * 对于样例输入2：小智一个小精灵都没法收服，皮卡丘也不会收到任何伤害，所以输出0 100
 * <p>
 * 问题转化：
 * 明显，精灵球和体力都是开销，而每个小精灵对应的价值都为1（只），所以这是一个二维费用背包问题。转化过来就是在使用n个精灵球且总价值最大时，R取最大值的情况。
 * 一维背包只花费背包的体积
 * 这个二维背包,
 * 花费:精灵球数量和皮卡丘体力值
 * 需要花两个东西
 * 价值:小精灵的数量
 * 属性是max
 * 状态定义:f[i,j,k]表示为:所有只从前i种物品种选,
 * 且花费1不超过j,花费2不超过k的选法的最大价值
 * 状态计算:选第i个物品和不选第i个物品
 * 不选第i个物品:f[i-1,j,k]选择范围变小,精灵球数量和体力值不变(体积不变)
 * 选择第i个物品:max( f[i-1,j-v1[i],k-v2[i]]+1 , f[i-1,j,k] )
 * 最多收服的小精灵数量:f[K,N,M]
 * 最少耗费体力怎么计算呢:f[K,N,m]==f[K,N,M]
 * 其实是01背包问题可以三维优化二维
 */
public class 二维费用背包收服小精灵 {
    public static void main(String[] args) {
        Scanner sc = new Scanner(System.in);
        N = sc.nextInt();
        M = sc.nextInt();
        n = sc.nextInt();
        for (int i = 1; i <= n; i++) {
            v1[i] = sc.nextInt();
            v2[i] = sc.nextInt();
        }
        two();
        for (int i = 1; i <= n; i++) {
            for (int j = 0; j <= N; j++) {
                for (int k = 0; k < M; k++) {
                    dp[i][j][k] = dp[i - 1][j][k];
                    if (j >= v1[i] && k >= v2[i]) {
                        dp[i][j][k] = Math.max(dp[i][j][k], dp[i - 1][j - v1[i]][k - v2[i]] + 1);
                    }
                }
            }
        }
        System.out.println(dp[n][N][M - 1]);
        int k = M - 1;
        while (k > 0 && dp[n][N][k - 1] == dp[n][N][M - 1]) k--;
        //如果收付k个精灵,只花费M-1体力也行,或者M-2...下不封顶,找最小的那个花费,
        System.out.println(M - k);
    }

    static void two() {
        for (int i = 1; i <= n; i++) {
            for (int j = N; j >= v1[i]; j--) {
                for (int k = M - 1; k >= v2[i]; k--) {
                    f[j][k] = Math.max(f[j - v1[i]][k - v2[i]] + 1, f[j][k]);
                }
            }
        }
        System.out.println(f[N][M - 1]);
        //因为
        int k = M - 1;
        while (k > 0 && f[N][k - 1] == f[N][M - 1]) k--;
        System.out.println(M - k);
    }

    static int N, M, n;
    static int[] v1 = new int[1000], v2 = new int[1000];
    static int[][][] dp = new int[110][1010][510];

    static int[][] f = new int[1010][510];
}
